/*
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        28.05.2004
 Description:    Programm zur Ansteuerung eines Standart LCD
				 (HD44870),(SED1278F) und kompatible
*/

#include "main.h"
#include "lcd.h"

void LCD_Init (void)
{	
	int a, b;
	char tmp = Init_LCD_Zeilen;
	//Set Port Direction Register to Output for LCD Databus und LCD Steuerbus
	LCD_Port_DDR = LCD_DataOutput+(1<<LCD_RS | 1<<LCD_RW | 1<<LCD_E);
	//Wait a short Time afer Power on
	for (a=0;a<10000;a++)
		{
		for (b=0;b<100;b++);
		}	
	Write_LCD (0x22,0);	//Init in 4 Bit Mode

	//Wait a short Time afer Power on
	for (a=0;a<10000;a++)
		{
		for (b=0;b<100;b++);
		}	

	Write_LCD (0x22,0);	//Init in 4 Bit Mode (Zur Sicherheit nochmal)	
	if (tmp == 1)
		{
		Write_LCD (0x20,0);	//1 Zeilen
		}
		else
		{
		Write_LCD (0x28,0);	//mehrere Zeilen
		}
	Write_LCD (0x0C,0);	//Display On
	Write_LCD (0x80,0);	//Set DD-Ram Adresse = 0
}

void Write_LCD (char Data,char CD)
{
	//Set Port Direction Register to Output for LCD Databus und LCD Steuerbus
	LCD_Port_DDR = LCD_DataOutput+(1<<LCD_RS | 1<<LCD_RW | 1<<LCD_E);
	//Soll ins Seuer oder Datenregister geschrieben werden?
	if (CD == 0)
		{
		LCD_Port_Write &=~ (1<<LCD_RS); //RS = 0 Steuerregister
		}
		else
		{
		LCD_Port_Write |= (1<<LCD_RS); //RS = 1 Dataregister
		}
	//Schreibsignal setzen
	LCD_Port_Write &=~ (1<<LCD_RW);//Zum Schreiben RW-Pin = Low
	//Schreiben der 1. 4Bit an das LCD Display
	LCD_Port_Write = (LCD_Port_Write&0xF0) + ((Data&0xF0)>>4); //Write Nibbel MSB
	LCD_Port_Write |= (1<<LCD_E);
	LCD_Port_Write &=~ (1<<LCD_E);	
	//Schreiben der 2. 4Bit an das LCD Display	
	LCD_Port_Write = (LCD_Port_Write&0xF0) + (Data&0x0F); //Write Nibbel LSB
	LCD_Port_Write |= (1<<LCD_E);
	LCD_Port_Write &=~ (1<<LCD_E);
	
	//loop_until_bit_is_clear(Read_LCD(0),BusyBit);	//Wartet bis LCD wieder bereit
	while ((Read_LCD(0)&(1<<BusyBit)) > 0) {}; //Wartet bis LCD wieder bereit
}

char Read_LCD (char CD)
{
	char Data;
	//Set Port Direction Register to Output for LCD Databus und LCD Steuerbus
	LCD_Port_DDR = LCD_DataInput+(1<<LCD_RS | 1<<LCD_RW | 1<<LCD_E);
	//Lesesignal setzen
	LCD_Port_Write |= (1<<LCD_RW); //Zum Lesen RW-Pin = High
	//Soll ins Seuer oder Datenregister geschrieben werden?
	if (CD == 0)
		{
		LCD_Port_Write &=~ (1<<LCD_RS); //RS = 0 Steuerregister
		}
		else
		{
		LCD_Port_Write |= (1<<LCD_RS); //RS = 1 Dataregister
		}
	LCD_Port_Write |= (1<<LCD_E);//Daten stehen an wenn Enable = High
	
	char a = 0; //Wartet ein bisschen bis daten anliegen
	while (a < 100)
		{
		a ++;
		}
		
	Data = (LCD_Port_Read&0x0F)<<4; //Lesen des 1. Nibble (MSB)
	LCD_Port_Write &=~ (1<<LCD_E);	
	
	LCD_Port_Write |= (1<<LCD_E);//Daten stehen an wenn Enable = High
	
	a = 0; //Wartet ein bisschen bis daten anliegen
	while (a < 100)
		{
		a ++;
		}

	Data += (LCD_Port_Read&0x0F); //Lesen des 2. Nibble (LSB)
	LCD_Port_Write &=~ (1<<LCD_E);
return(Data);
}

void LCD_Print (char zeile,char spalte,char *Buffer,...)
{
	va_list ap;
	va_start (ap, Buffer);	
	
	int format_flag;
	char str_buffer[10];
	char str_null_buffer[10];
	char move = 0;
	char Base = 0;
	int tmp = 0;
	
	//Berechnet Adresse fr die Zeile und schreibt sie ins DD-Ram
	if (zeile >= Init_LCD_Zeilen) //wurden mehr Zeilen angegeben als Initialisiert
		{
		zeile = Init_LCD_Zeilen - 1;
		}
	if (zeile == 0)
		{
		zeile = 0x80;
		}
	if (zeile == 1)
		{
		zeile = 0xC0;
		}
	if (zeile == 2)
		{
		zeile = 0x90;
		}
	if (zeile == 3)
		{
		zeile = 0xd0;
		}
	zeile += spalte;
	Write_LCD (zeile,0);
	
	
	//Ausgabe der Zeichen
	while (*Buffer != 0)
		{
		if (*Buffer == '%')
			{
			*Buffer++;
			if (isdigit(*Buffer)>0)
				{
				str_null_buffer[0] = *Buffer++;
				str_null_buffer[1] = '\0';
				move = atoi(str_null_buffer);
				}
			switch (format_flag = *Buffer++)
				{
				case 'b':
					Base = 2;
					goto ConversionLoop;
				case 'c':
					//Int to char
					format_flag = va_arg(ap,int);
					Write_LCD (format_flag++,1); 
					break;
				case 'i':
					Base = 10;
					goto ConversionLoop;
				case 'o':
					Base = 8;
					goto ConversionLoop;
				case 'x':
					Base = 16;
					//****************************
					ConversionLoop:
					//****************************
					itoa(va_arg(ap,int),str_buffer,Base);
					int b=0;
					while (str_buffer[b++] != 0){};
					b--;
					if (b<move)
						{
						move -=b;
						for (tmp = 0;tmp<move;tmp++)
							{
							str_null_buffer[tmp] = '0';
							}
						//tmp ++;
						str_null_buffer[tmp] = '\0';
						strcat(str_null_buffer,str_buffer);
						strcpy(str_buffer,str_null_buffer);
						}
					LCD_print_str (str_buffer);
					move =0;
					break;
				}
			
			}	
		else
			{
			Write_LCD (*Buffer++,1);
			}
		}
	va_end(ap);
}

void LCD_print_str (char *Buffer)
{
	while (*Buffer != 0)
		{
		Write_LCD (*Buffer++,1);
		};
};

void LCD_Clear (void)
{
	Write_LCD (1,0); //Clear Display
	Write_LCD (0x80,0);	//Set DD-Ram Adresse = 0
}



